/*****************************************************************************
 *                                                                           *
 *        **********                                                         *
 *       ************                                                        *
 *      ***        ***                                                       *
 *      ***   +++   ***                                                      *
 *      ***   + +   ***                                                      *
 *      ***   +                    CHIPCON CC1010 EXAMPLE PROGRAM            *
 *      ***   + +   ***                      CC1010 CHAT                     *
 *      ***   +++   ***                                                      *
 *      ***       ***                                                        *
 *       ***********                                                         *
 *        *********                                                          *
 *                                                                           *
 *****************************************************************************
 * Author:              JOL                                                  *
 *****************************************************************************
 * Revision history:                                                         *
 *                                                                           *
 * $Log: chatKeyExchange.c,v $
 * Revision 1.1  2003/07/29 11:16:29  tos
 * Initial version in CVS.
 *
 *
 *                                                                           *
 ****************************************************************************/

/*****************************************************************************
 *****************************************************************************
 *************    CLIENT/CLIENT KEY EXCHANGE (DIFFIE-HELLMAN)    *************
 *****************************************************************************
 ****************************************************************************/
#include "chat.h"


//----------------------------------------------------------------------------
//  bool diffieHellmanUp(byte destAddress, byte xdata *pKeyMaterial)
//
//  Description:
//      Prepare and transmit the Diffie-Hellman "key material" to destAddress,
//		then wait for the returned key material from destAddress.
//		Create a 7-byte DES key for encrypted private communication with d.A.  
//
//	Arguments:
//		byte destAddress
//			The address we're creating the key for
//
//	Returned value:
//		bool
//			TRUE if OK, FALSE otherwise
//----------------------------------------------------------------------------
/*
byte tull = 0x12;
byte* destAdd = &tull;
byte save[6];
byte written[6] = {0,0,0,0,0,0};
*/

bool diffieHellmanUp(byte destAddress) {
	byte n;
	byte tempKey[8];
	CLIENT_INFO xdata *pDestCI;

	if (pDestCI = getClientInfo(destAddress)) {

		// Transmit and receive settings
		TXI.destination = destAddress;
		TXI.flags = SPP_ACK_REQ | MSG_DIFFIE_HELLMAN_UP;
		TXI.pDataBuffer = pAdminTXBuffer;
		TXI.dataLen = 8;
		RXI.pDataBuffer = pRXBuffer;

		// Prepare our own Diffie-Hellman data (DUMMY METHOD)
		srand(pMyCI->address * 0x0101);
		for (n = 0; n < 8; n++) {
			TXI.pDataBuffer[n] = rand();
		}

		// Transmit the key material

		fhsppSend(&TXI);
		while (fhsppStatus() != SPP_IDLE_MODE);
		if (TXI.status != SPP_TX_FINISHED) {
			return FALSE;
		}

		// Receive the returned key material
restartRX:
		fhsppReceive(&RXI);
		sppSettings.rxTimeout = SHORT_RX_TIMEOUT;
		while (fhsppStatus() != SPP_IDLE_MODE);
		if (RXI.status != SPP_RX_FINISHED) {
			return FALSE;
		} else {
			if (RXI.source != destAddress) {
				goto restartRX;
			}
		}

		// Generate the DES key and place it in the CI table (DUMMY METHOD)
		for (n = 0; n < 8; n++) {
			tempKey[n] = TXI.pDataBuffer[n] ^ RXI.pDataBuffer[n];
		}
		memcpy(pDestCI->pDESKey, tempKey, CI_KEY_LENGTH);
		pDestCI->isDESKeyValid = TRUE;

//		debu(sprintf(mess,"diffieHellmanUp completed"));
		return TRUE;

	} else {
		return FALSE;
	}

} // diffieHellmanUp




//----------------------------------------------------------------------------
//  bool diffieHellmanDown(CLIENT_INFO xdata *pSourceCI, byte xdata *pKeyM...)
//
//  Description:
//      We've just received key material from another client. Prepare and 
//		transmit our Diffie-Hellman "key material" to sourceAddress,
//		Create a 7-byte DES key for encrypted private communication with s.A.  
//
//	Arguments:
//		CLIENT_INFO xdata *pSourceCI
//			The address we're creating the key for
//		byte xdata *pKeyMaterial
//			Key material from pSourceCI
//----------------------------------------------------------------------------
void diffieHellmanDown(CLIENT_INFO xdata *pSourceCI, byte xdata *pKeyMaterial) {
	byte n;
	byte tempKey[8];

	// Transmit settings
	TXI.destination = pSourceCI->address;
	TXI.flags = SPP_ACK_REQ | MSG_DIFFIE_HELLMAN_DOWN; //MSG_DIFFIE_HELLMAN_DOWN ? // was: MSG_DIFFIE_HELLMAN_UP
	TXI.pDataBuffer = pAdminTXBuffer;
	TXI.dataLen = 8;
	
	// Prepare our own Diffie-Hellman data (DUMMY METHOD)
	srand(pMyCI->address * 0x0101);
	for (n = 0; n < 8; n++) {
		TXI.pDataBuffer[n] = rand();
	}

	// Transmit the key material
	fhsppSend(&TXI);
	while (fhsppStatus() != SPP_IDLE_MODE);
	if (TXI.status != SPP_TX_FINISHED) {
		return;
	}

	// Generate DES key (DUMMY METHOD)
	for (n = 0; n < 8; n++) {
		tempKey[n] = TXI.pDataBuffer[n] ^ pKeyMaterial[n];
	}
	memcpy(pSourceCI->pDESKey, tempKey, CI_KEY_LENGTH);
	pSourceCI->isDESKeyValid = TRUE;
} // diffieHellmanDown
